@@ -18,8 +18,17 @@
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  *      MA 02110-1301, USA.
  */
- 
+
+#ifndef CALLTIP_H
+#define CALLTIP_H
+
+#include <glib.h>
+
+#include "debug_module.h"
+
 #define MAX_CALLTIP_LENGTH 140
 #define MAX_CALLTIP_HEIGHT 20
 
 GString* get_calltip_line(variable *var, gboolean firstline);
+
+#endif /* guard */
