@@ -30,6 +30,7 @@
 #include "gwh-utils.h"
 #include "gwh-settings.h"
 #include "gwh-keybindings.h"
+#include "gwh-plugin.h"
 
 
 #if ! GTK_CHECK_VERSION (2, 18, 0)
@@ -48,6 +49,12 @@
   (GTK_WIDGET_MAPPED ((w)))
 # endif /* defined (gtk_widget_get_mapped) */
 #endif /* GTK_CHECK_VERSION (2, 20, 0) */
+#if GTK_CHECK_VERSION (3, 0, 0)
+/* alias GtkObject, we implement the :destroy signal */
+# define GtkObject          GtkWidget
+# define GtkObjectClass     GtkWidgetClass
+# define GTK_OBJECT_CLASS   GTK_WIDGET_CLASS
+#endif /* GTK_CHECK_VERSION (3, 0, 0) */
 
 
 struct _GwhBrowserPrivate
@@ -74,6 +81,9 @@ struct _GwhBrowserPrivate
   GtkToolItem  *item_cancel;
   GtkToolItem  *item_reload;
   GtkToolItem  *item_inspector;
+  
+  GtkWidget    *statusbar;
+  gchar        *hovered_link;
 };
 
 enum {
@@ -97,6 +107,9 @@ G_DEFINE_TYPE_WITH_CODE (GwhBrowser, gwh_browser, GTK_TYPE_VBOX,
                          G_IMPLEMENT_INTERFACE (GTK_TYPE_ORIENTABLE, NULL))
 
 
+static void       inspector_set_detached      (GwhBrowser *self,
+                                               gboolean    detached);
+
 
 static void
 set_location_icon (GwhBrowser  *self,
@@ -185,6 +198,17 @@ on_settings_inspector_window_geometry_notify (GObject    *object,
   g_free (geometry);
 }
 
+static void
+on_settings_inspector_detached_notify (GObject    *object,
+                                       GParamSpec *pspec,
+                                       GwhBrowser *self)
+{
+  gboolean detached;
+  
+  g_object_get (object, pspec->name, &detached, NULL);
+  inspector_set_detached (self, detached);
+}
+
 static void
 on_settings_wm_windows_skip_taskbar_notify (GObject    *object,
                                             GParamSpec *pspec,
@@ -261,11 +285,39 @@ inspector_show_window (GwhBrowser *self)
   }
 }
 
+static void
+inspector_set_detached (GwhBrowser *self,
+                        gboolean    detached)
+{
+  if (detached != INSPECTOR_DETACHED (self)) {
+    if (detached) {
+      gtk_widget_reparent (self->priv->inspector_view,
+                           self->priv->inspector_window);
+      if (INSPECTOR_VISIBLE (self)) {
+        inspector_show_window (self);
+      }
+    } else {
+      gtk_widget_reparent (self->priv->inspector_view, self->priv->paned);
+      inspector_hide_window (self);
+    }
+    g_object_set (self->priv->settings, "inspector-detached", detached, NULL);
+  }
+}
+
 static WebKitWebView *
 on_inspector_inspect_web_view (WebKitWebInspector *inspector,
                                WebKitWebView      *view,
                                GwhBrowser         *self)
 {
+  if (self->priv->inspector_web_view) {
+    gtk_widget_destroy (self->priv->inspector_web_view);
+  }
+  
+  self->priv->inspector_web_view = webkit_web_view_new ();
+  gtk_widget_show (self->priv->inspector_web_view);
+  gtk_container_add (GTK_CONTAINER (self->priv->inspector_view),
+                     self->priv->inspector_web_view);
+  
   return WEBKIT_WEB_VIEW (self->priv->inspector_web_view);
 }
 
@@ -300,9 +352,7 @@ static gboolean
 on_inspector_detach_window (WebKitWebInspector *inspector,
                             GwhBrowser         *self)
 {
-  gtk_widget_reparent (self->priv->inspector_view,
-                       self->priv->inspector_window);
-  inspector_show_window (self);
+  inspector_set_detached (self, TRUE);
   
   return TRUE;
 }
@@ -311,8 +361,7 @@ static gboolean
 on_inspector_attach_window (WebKitWebInspector *inspector,
                             GwhBrowser         *self)
 {
-  gtk_widget_reparent (self->priv->inspector_view, self->priv->paned);
-  inspector_hide_window (self);
+  inspector_set_detached (self, FALSE);
   
   return TRUE;
 }
@@ -476,7 +525,7 @@ on_web_view_populate_popup (WebKitWebView *view,
   #define ADD_SEPARATOR(menu) \
     item = gtk_separator_menu_item_new (); \
     gtk_widget_show (item); \
-    gtk_menu_append (menu, item)
+    gtk_menu_shell_append (GTK_MENU_SHELL (menu), item)
   
   ADD_SEPARATOR (menu);
   
@@ -485,23 +534,23 @@ on_web_view_populate_popup (WebKitWebView *view,
   item = gtk_menu_item_new_with_mnemonic (_("_Zoom"));
   gtk_widget_show (item);
   gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), submenu);
-  gtk_menu_append (menu, item);
+  gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
   /* zoom in */
   item = gtk_image_menu_item_new_from_stock (GTK_STOCK_ZOOM_IN, NULL);
   g_signal_connect_swapped (item, "activate",
                             G_CALLBACK (webkit_web_view_zoom_in), view);
-  gtk_menu_append (GTK_MENU (submenu), item);
+  gtk_menu_shell_append (GTK_MENU_SHELL (submenu), item);
   /* zoom out */
   item = gtk_image_menu_item_new_from_stock (GTK_STOCK_ZOOM_OUT, NULL);
   g_signal_connect_swapped (item, "activate",
                             G_CALLBACK (webkit_web_view_zoom_out), view);
-  gtk_menu_append (GTK_MENU (submenu), item);
+  gtk_menu_shell_append (GTK_MENU_SHELL (submenu), item);
   /* zoom 1:1 */
   ADD_SEPARATOR (submenu);
   item = gtk_image_menu_item_new_from_stock (GTK_STOCK_ZOOM_100, NULL);
   g_signal_connect (item, "activate",
                     G_CALLBACK (on_item_zoom_100_activate), self);
-  gtk_menu_append (GTK_MENU (submenu), item);
+  gtk_menu_shell_append (GTK_MENU_SHELL (submenu), item);
   /* full content zoom */
   ADD_SEPARATOR (submenu);
   item = gtk_check_menu_item_new_with_mnemonic (_("Full-_content zoom"));
@@ -509,7 +558,7 @@ on_web_view_populate_popup (WebKitWebView *view,
                                   webkit_web_view_get_full_content_zoom (view));
   g_signal_connect (item, "activate",
                     G_CALLBACK (on_item_full_content_zoom_activate), self);
-  gtk_menu_append (GTK_MENU (submenu), item);
+  gtk_menu_shell_append (GTK_MENU_SHELL (submenu), item);
   /* show zoom sumbenu */
   gtk_widget_show_all (submenu);
   
@@ -519,7 +568,7 @@ on_web_view_populate_popup (WebKitWebView *view,
   g_signal_connect (item, "activate",
                     G_CALLBACK (on_item_flip_orientation_activate), self);
   gtk_widget_show (item);
-  gtk_menu_append (menu, item);
+  gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
   if (! INSPECTOR_VISIBLE (self) || INSPECTOR_DETACHED (self)) {
     gtk_widget_set_sensitive (item, FALSE);
   }
@@ -605,6 +654,8 @@ gwh_browser_finalize (GObject *object)
     g_object_unref (self->priv->default_icon);
   }
   g_object_unref (self->priv->settings);
+  g_object_unref (self->priv->statusbar);
+  g_free (self->priv->hovered_link);
   
   G_OBJECT_CLASS (gwh_browser_parent_class)->finalize (object);
 }
@@ -839,17 +890,80 @@ create_inspector_window (GwhBrowser *self)
                     G_CALLBACK (on_settings_wm_windows_skip_taskbar_notify), self);
   g_signal_connect (self->priv->settings, "notify::wm-secondary-windows-type",
                     G_CALLBACK (on_settings_wm_windows_type_notify), self);
-  gtk_container_add (GTK_CONTAINER (self->priv->inspector_window),
-                     self->priv->inspector_view);
   
   return self->priv->inspector_window;
 }
 
+static guint
+get_statusbar_context_id (GtkStatusbar *statusbar)
+{
+  static guint id = 0;
+  
+  if (id == 0) {
+    id = gtk_statusbar_get_context_id (statusbar, "gwh-browser-hovered-link");
+  }
+  
+  return id;
+}
+
+static void
+on_web_view_hovering_over_link (WebKitWebView *view,
+                                gchar         *title,
+                                gchar         *uri,
+                                GwhBrowser    *self)
+{
+  GtkStatusbar *statusbar = GTK_STATUSBAR (self->priv->statusbar);
+  
+  if (self->priv->hovered_link) {
+    gtk_statusbar_pop (statusbar, get_statusbar_context_id (statusbar));
+    g_free (self->priv->hovered_link);
+    self->priv->hovered_link = NULL;
+  }
+  if (uri && *uri) {
+    self->priv->hovered_link = g_strdup (uri);
+    gtk_statusbar_push (statusbar, get_statusbar_context_id (statusbar),
+                        self->priv->hovered_link);
+  }
+}
+
+static void
+on_web_view_leave_notify_event (GtkWidget        *widget,
+                                GdkEventCrossing *event,
+                                GwhBrowser       *self)
+{
+  if (self->priv->hovered_link) {
+    GtkStatusbar *statusbar = GTK_STATUSBAR (self->priv->statusbar);
+    
+    gtk_statusbar_pop (statusbar, get_statusbar_context_id (statusbar));
+  }
+}
+
+static void
+on_web_view_enter_notify_event (GtkWidget        *widget,
+                                GdkEventCrossing *event,
+                                GwhBrowser       *self)
+{
+  if (self->priv->hovered_link) {
+    GtkStatusbar *statusbar = GTK_STATUSBAR (self->priv->statusbar);
+    
+    gtk_statusbar_push (statusbar, get_statusbar_context_id (statusbar),
+                        self->priv->hovered_link);
+  }
+}
+
+static void
+on_web_view_realize (GtkWidget  *widget,
+                     GwhBrowser *self)
+{
+  gtk_widget_add_events (widget, GDK_LEAVE_NOTIFY_MASK | GDK_ENTER_NOTIFY_MASK);
+}
+
 static void
 gwh_browser_init (GwhBrowser *self)
 {
   GtkWidget          *scrolled;
   WebKitWebSettings  *wkws;
+  gboolean            inspector_detached;
   
   self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, GWH_TYPE_BROWSER,
                                             GwhBrowserPrivate);
@@ -861,6 +975,9 @@ gwh_browser_init (GwhBrowser *self)
   g_object_set (wkws, "enable-developer-extras", TRUE, NULL);
   
   self->priv->settings = gwh_settings_get_default ();
+  g_object_get (self->priv->settings,
+                "inspector-detached", &inspector_detached,
+                NULL);
   
   self->priv->toolbar = create_toolbar (self);
   self->priv->paned = gtk_vpaned_new ();
@@ -878,12 +995,22 @@ gwh_browser_init (GwhBrowser *self)
   self->priv->inspector_view = gtk_scrolled_window_new (NULL, NULL);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (self->priv->inspector_view),
                                   GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-  self->priv->inspector_web_view = webkit_web_view_new ();
-  gtk_widget_show (self->priv->inspector_web_view);
-  gtk_container_add (GTK_CONTAINER (self->priv->inspector_view),
-                     self->priv->inspector_web_view);
+  self->priv->inspector_web_view = NULL;
   
   self->priv->inspector_window = create_inspector_window (self);
+  gtk_container_add (GTK_CONTAINER (inspector_detached
+                                    ? self->priv->inspector_window
+                                    : self->priv->paned),
+                     self->priv->inspector_view);
+  
+  self->priv->statusbar = ui_lookup_widget (geany->main_widgets->window, "statusbar");
+  if (self->priv->statusbar) {
+    g_object_ref (self->priv->statusbar);
+  } else {
+    /* in the unlikely case we can't get the Geany statusbar, fake one */
+    self->priv->statusbar = gtk_statusbar_new ();
+  }
+  self->priv->hovered_link = NULL;
   
   g_signal_connect (self, "notify::orientation",
                     G_CALLBACK (on_orientation_notify), self);
@@ -914,6 +1041,14 @@ gwh_browser_init (GwhBrowser *self)
                     G_CALLBACK (on_web_view_populate_popup), self);
   g_signal_connect (G_OBJECT (self->priv->web_view), "scroll-event",
                     G_CALLBACK (on_web_view_scroll_event), self);
+  g_signal_connect (G_OBJECT (self->priv->web_view), "hovering-over-link",
+                    G_CALLBACK (on_web_view_hovering_over_link), self);
+  g_signal_connect (G_OBJECT (self->priv->web_view), "leave-notify-event",
+                    G_CALLBACK (on_web_view_leave_notify_event), self);
+  g_signal_connect (G_OBJECT (self->priv->web_view), "enter-notify-event",
+                    G_CALLBACK (on_web_view_enter_notify_event), self);
+  g_signal_connect_after (self->priv->web_view, "realize",
+                          G_CALLBACK (on_web_view_realize), self);
   
   g_signal_connect (self->priv->web_view, "key-press-event",
                     G_CALLBACK (gwh_keybindings_handle_event), self);
@@ -926,6 +1061,8 @@ gwh_browser_init (GwhBrowser *self)
                     G_CALLBACK (on_settings_browser_last_uri_notify), self);
   g_signal_connect (self->priv->settings, "notify::browser-orientation",
                     G_CALLBACK (on_settings_browser_orientation_notify), self);
+  g_signal_connect (self->priv->settings, "notify::inspector-detached",
+                    G_CALLBACK (on_settings_inspector_detached_notify), self);
   g_signal_connect (self->priv->settings, "notify::inspector-window-geometry",
                     G_CALLBACK (on_settings_inspector_window_geometry_notify), self);
 }
